"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Registry = exports.BurnsCache = exports.StakesCache = exports.RegistrationsCache = exports.Burn = exports.Stake = exports.Registration = void 0;
// Local imports
const sdk_utils_1 = require("@tornado/sdk-utils");
const sdk_data_1 = require("@tornado/sdk-data");
const sdk_chain_1 = require("@tornado/sdk-chain");
// @ts-ignore
const pouchdb_collate_1 = require("pouchdb-collate");
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ IMPLEMENTATIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
class Registration extends sdk_data_1.Docs.Base {
    ensName;
    address;
    relayer;
    stakedAmount;
    constructor(obj) {
        const blockNumber = obj['blockNumber'];
        const ensName = obj['args']['ensName'];
        super((0, pouchdb_collate_1.toIndexableString)([blockNumber, ensName]));
        this.ensName = ensName;
        this.relayer = obj['args']['relayer'];
        this.address = obj['args']['address'];
        this.stakedAmount = obj['args']['stakedAmount'].toString();
    }
}
exports.Registration = Registration;
class Stake extends sdk_data_1.Docs.Base {
    relayer;
    amountStakeAdded;
    constructor(obj) {
        const blockNumber = obj['blockNumber'];
        super((0, pouchdb_collate_1.toIndexableString)([blockNumber]));
        this.relayer = obj['args']['relayer'];
        this.amountStakeAdded = obj['args']['amountStakeAdded'].toString();
    }
}
exports.Stake = Stake;
class Burn extends sdk_data_1.Docs.Base {
    relayer;
    amountBurned;
    constructor(obj) {
        const blockNumber = obj['blockNumber'];
        super((0, pouchdb_collate_1.toIndexableString)([blockNumber]));
        this.relayer = obj['args']['relayer'];
        this.amountBurned = obj['args']['amountBurned'].toString();
    }
}
exports.Burn = Burn;
class RegistrationsCache extends sdk_data_1.Cache.Syncable {
    buildDoc(response) {
        return new Registration(response);
    }
    getErrorHandlers() {
        return [sdk_chain_1.syncErrorHandler];
    }
    getCallbacks(registry) {
        return [
            (fromBlock, toBlock) => {
                return registry.queryFilter(registry.filters['RelayerRegistered(bytes32,string,address,uint256)'](null, null, null, null), fromBlock, toBlock);
            }
        ];
    }
}
exports.RegistrationsCache = RegistrationsCache;
class StakesCache extends sdk_data_1.Cache.Syncable {
    buildDoc(response) {
        return new Stake(response);
    }
    getErrorHandlers() {
        return [sdk_chain_1.syncErrorHandler];
    }
    getCallbacks(registry) {
        return [
            (fromBlock, toBlock) => {
                return registry.queryFilter(registry.filters['StakeAddedToRelayer(address,uint256)'](null, null), fromBlock, toBlock);
            }
        ];
    }
}
exports.StakesCache = StakesCache;
class BurnsCache extends sdk_data_1.Cache.Syncable {
    buildDoc(response) {
        return new Burn(response);
    }
    getErrorHandlers() {
        return [sdk_chain_1.syncErrorHandler];
    }
    getCallbacks(registry) {
        return [
            (fromBlock, toBlock) => {
                return registry.queryFilter(registry.filters['StakeBurned(address,uint256)'](null, null), fromBlock, toBlock);
            }
        ];
    }
}
exports.BurnsCache = BurnsCache;
class Registry extends sdk_chain_1.Synchronizer {
    _chain;
    get chain() {
        this._checkProvider('chain');
        return this._chain;
    }
    constructor() {
        super();
        this.caches = new Map();
    }
    _checkProvider(parentCallName) {
        if (!this._chain)
            throw sdk_utils_1.ErrorUtils.getError('Core.' + parentCallName + ': you must first connect a provider!');
    }
    async connect(provider) {
        if (!this._chain)
            this._chain = new sdk_chain_1.Chain(provider);
        else
            this._chain.provider = provider;
        await this._chain.fetchChainData();
    }
    getContract() {
        return sdk_chain_1.Contracts.getRegistry(this.chain.provider);
    }
    async syncRegistrations(networkId, options) {
        await this._sync('RelayerRegistered', networkId, options);
    }
    async syncStakes(networkId, options) {
        await this._sync('StakeAddedToRelayer', networkId, options);
    }
    async syncBurns(networkId, options) {
        await this._sync('StakeBurned', networkId, options);
    }
    async _sync(event, networkId, options) {
        networkId = networkId ?? '1';
        if (this.chain.id !== +networkId)
            throw sdk_utils_1.ErrorUtils.getError('Registry._sync: provider bound to wrong chain!');
        let registry = this.getContract(), needsFilter = typeof event !== 'string', filter = event, emitterName, cache;
        options = options ?? {};
        options.startBlock = options.startBlock ?? (await sdk_data_1.Onchain.getRegistryDeployBlockNum('' + networkId));
        if (event === 'RelayerRegistered') {
            cache = this.loadRegistrations(networkId, options);
            emitterName = 'registration';
            if (needsFilter)
                filter = registry.filters.RelayerRegistered(null, null, null, null);
        }
        else if (event === 'StakeAddedToRelayer') {
            cache = this.loadStakes(networkId, options);
            emitterName = 'stake';
            if (needsFilter)
                filter = registry.filters.StakeAddedToRelayer(null, null);
        }
        else if (event === 'StakeBurned') {
            cache = this.loadBurns(networkId, options);
            emitterName = 'burn';
            if (needsFilter)
                filter = registry.filters['StakeBurned(address,uint256)'](null, null);
        }
        else
            throw sdk_utils_1.ErrorUtils.getError(`Registry._sync: such an event ${event} is not supported`);
        await this.sync(emitterName, filter, registry, cache, options);
    }
    loadRegistrations(networkId, options) {
        networkId = networkId ?? '1';
        return this.loadCache('Registrations' + networkId, options);
    }
    loadStakes(networkId, options) {
        networkId = networkId ?? '1';
        return this.loadCache('Stakes' + networkId, options);
    }
    loadBurns(networkId, options) {
        networkId = networkId ?? '1';
        return this.loadCache('Burns' + networkId, options);
    }
    loadCache(name, options) {
        let constructor, cache = super.loadCache(name);
        if (!cache) {
            const regexp = /([A-Za-z]+)([0-9]+)/;
            const matches = name.match(regexp);
            if (!matches)
                throw sdk_utils_1.ErrorUtils.getError(`Registry.loadCache: name ${name} has wrong format for cache`);
            if (matches.length === 2)
                name += '1';
            if (matches[1] === 'Registrations') {
                constructor = (name, options) => new RegistrationsCache(name, options);
            }
            else if (matches[1] === 'Burns') {
                constructor = (name, options) => new BurnsCache(name, options);
            }
            else if (matches[1] === 'Stakes') {
                constructor = (name, options) => new StakesCache(name, options);
            }
            else
                throw sdk_utils_1.ErrorUtils.getError(`Registry.loadCache: there exists no such event ${matches[1]}`);
            cache = constructor(name, options);
            this.caches.set(name, cache);
        }
        return cache;
    }
    async exportRegistrationsZip(networkId, outDirPath, close, options) {
        networkId = networkId ?? '1';
        await this._exportCacheZip('Registrations' + networkId, outDirPath, close, options);
    }
    async exportStakesZip(networkId, outDirPath, close, options) {
        networkId = networkId ?? '1';
        await this._exportCacheZip('Stakes' + networkId, outDirPath, close, options);
    }
    async exportBurnsZip(networkId, outDirPath, close, options) {
        networkId = networkId ?? '1';
        await this._exportCacheZip('Burns' + networkId, outDirPath, close, options);
    }
    async exportRegistrationsJson(networkId, outDirPath, close, options) {
        networkId = networkId ?? '1';
        await this._exportCacheJson('Registrations' + networkId, outDirPath, close, options);
    }
    async exportStakesJson(networkId, outDirPath, close, options) {
        networkId = networkId ?? '1';
        await this._exportCacheJson('Stakes' + networkId, outDirPath, close, options);
    }
    async exportBurnsJson(networkId, outDirPath, close, options) {
        networkId = networkId ?? '1';
        await this._exportCacheJson('Burns' + networkId, outDirPath, close, options);
    }
    async _exportCacheZip(cacheName, outDirPath, close, options) {
        const cache = new sdk_data_1.Cache.Base(cacheName, options);
        await cache.zip(outDirPath, close);
        if (close === true)
            this.caches.delete(cacheName);
    }
    async _exportCacheJson(cacheName, outDirPath, close, options) {
        const cache = new sdk_data_1.Cache.Base(cacheName, options);
        await cache.jsonify(outDirPath);
        if (close === true)
            this.caches.delete(cacheName);
    }
    async _populateSyncOptions(options) {
        options.targetBlock = options.targetBlock ?? (await this.chain.latestBlockNum());
        return super._populateSyncOptions(options);
    }
}
exports.Registry = Registry;
//# sourceMappingURL=index.js.map